/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2020 Databases and Information Systems Group TU Dortmund           *
 * Visit us at                                                                  *
 *             http://dbis.cs.tu-dortmund.de/cms/en/home/                       *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *                                                                              *
 * Authors (alphabetical list):                                                 *
 *          Maximilian Berens   <maximilian.berens@tu-dortmund.de>              *
 *          Roland Kühn         <roland.kuehn@cs.tu-dortmund.de>                *
 *          Jan Mühlig          <jan.muehlig@tu-dortmund.de>                    *
 *------------------------------------------------------------------------------*
 */

#pragma once

#include "unary_operator.h"
#include <disk/buffer_manager.h>
#include <set>
#include <statistic/system_statistics.h>
#include <table/table.h>
#include <table/table_disk_manager.h>
#include <table/tuple.h>

namespace beedb::execution
{
/**
 * Inserts all tuples provided by the children operator.
 * The child may be a tuple buffer or a subquery.
 */
class InsertOperator final : public UnaryOperator
{
  public:
    InsertOperator(disk::BufferManager &buffer_manager, table::TableDiskManager &table_disk_manager,
                   statistic::SystemStatistics &statistics, table::Table &table);
    virtual ~InsertOperator() = default;

    virtual void open();
    virtual util::optional<table::Tuple> next();
    virtual void close();

    virtual const table::Schema &schema() const
    {
        return _schema;
    }

  private:
    const table::Schema _schema;
    disk::BufferManager &_buffer_manager;
    table::TableDiskManager &_table_disk_manager;
    statistic::SystemStatistics &_statistics;
    table::Table &_table;
    disk::Page::page_id _last_pinned_page = disk::Page::INVALID_PAGE_ID;
};
} // namespace beedb::execution