/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2020 Databases and Information Systems Group TU Dortmund           *
 * Visit us at                                                                  *
 *             http://dbis.cs.tu-dortmund.de/cms/en/home/                       *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *                                                                              *
 * Authors (alphabetical list):                                                 *
 *          Maximilian Berens   <maximilian.berens@tu-dortmund.de>              *
 *          Roland Kühn         <roland.kuehn@cs.tu-dortmund.de>                *
 *          Jan Mühlig          <jan.muehlig@tu-dortmund.de>                    *
 *------------------------------------------------------------------------------*
 */

#pragma once

#include <map>

#include "database.h"
#include "plan/logical/plan.h"

namespace beedb::plan::logical
{

class Optimizer
{
    using PredicateMap = std::map<logical::NodeIDType, std::reference_wrapper<const expression::Predicate>>;

    // map, that holds references to a specific type of operator (for convenient iteration and typecheck)
    template <typename OperatorType>
    using OperatorMap = std::map<logical::NodeIDType, std::reference_wrapper<const OperatorType>>;

  public:
    Optimizer(const beedb::Database &database, const logical::Plan &unoptimized_plan);

    const logical::Plan optimize();

  private:
    void optimization_force_hash_join();

    /**
     * @brief fill_access_maps addes references to the Operator<> maps (members of this class).
     */
    void fill_access_maps();
    /// Members:

    const beedb::Database &_database;
    const beedb::Config &_config;
    const logical::Plan &_plan;

    /// Access structures:

    const NodeIDType _plan_root;
    PredicateMap _plan_join_predicate_map;
    PredicateMap _plan_selection_predicate_map;

    OperatorMap<expression::TableOperator> _plan_table_map;
    OperatorMap<expression::JoinOperator> _plan_join_map;
    OperatorMap<expression::SelectionOperator> _plan_selection_map;
};

} // namespace beedb::plan::logical