/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2020 Databases and Information Systems Group TU Dortmund           *
 * Visit us at                                                                  *
 *             http://dbis.cs.tu-dortmund.de/cms/en/home/                       *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *                                                                              *
 * Authors (alphabetical list):                                                 *
 *          Maximilian Berens   <maximilian.berens@tu-dortmund.de>              *
 *          Roland Kühn         <roland.kuehn@cs.tu-dortmund.de>                *
 *          Jan Mühlig          <jan.muehlig@tu-dortmund.de>                    *
 *------------------------------------------------------------------------------*
 */

#include <algorithm>
#include <io/result_output_formatter.h>
#include <string>

using namespace beedb::io;

void ResultOutputFormatter::header(const beedb::table::Schema &schema)
{
    std::vector<std::string> header;
    std::transform(schema.attributes().cbegin(), schema.attributes().cend(), std::back_inserter(header),
                   [](const auto &attribute) -> std::string { return attribute; });
    this->_table.header(std::move(header));
}

void ResultOutputFormatter::push_back(const beedb::table::Tuple &tuple)
{
    if (tuple.has_data())
    {
        std::vector<std::string> row;
        std::transform(tuple.schema().column_order().cbegin(), tuple.schema().column_order().cend(),
                       std::back_inserter(row), [&tuple](const auto index) -> std::string { return tuple.get(index); });

        this->_table.push_back(std::move(row));
        this->_count_tuples++;
    }
}

void ResultOutputFormatter::push_back(const std::vector<table::Tuple> &tuples)
{
    for (const auto &row : tuples)
    {
        this->push_back(row);
    }
}

namespace beedb::io
{

std::ostream &operator<<(std::ostream &stream, const ResultOutputFormatter &result_output_formatter)
{
    return stream << result_output_formatter._table;
}
} // namespace beedb::io